#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import sys
import nijmetrics
		
if __name__ == '__main__':

	if '--help' in sys.argv:
		
		print "This script takes a pickled file from 'extract_nij_metrics' and generates various outputs out of it. The kind of output depends on the first argument."
		print ""
		print "Syntax:"
		print os.path.split(sys.argv[0])[1]+" ACTION INPUT_FILE"
		print ""
		print "Options:"
		print " -t FILE: tee (duplicate) the output to the specified logfile."
		print ""
		print "Actions:"
		print " images: generates 4 .PNG files per exercise: one for each quality level (good, medium and low) and an overlaid version (consensus). Take care: the consensus image is *not* appropriate for numerical analysis since the colors from PiAnoS are used and these do not map to the 3 RGB channels. Files for each quality level are only slightly better: the pixel intensity is normalized to the [0-255] range."
		print " csv: generates many .CSV files: for each exercise, one consensus and one file per user and quality level (so for 5 exercises with 100 users this is 5x(100x3+1)=1505 files)"
		print " coherency: generates the .CSV files containing the coherency metrics"
		sys.exit(0)
	
	tee = None
	args = sys.argv[1:]
	try:
		idx = args.index('-t')
		if len(args) <= idx:
			print "ERROR: -t option requires an argument"
			sys.exit(1)
		tee = args[idx+1]
		args = args[:idx] + args[idx+2:]
	except ValueError:
		pass
		
	if len(args) < 1:
		print "ERROR: you must specify an action as first argument"
		sys.exit(1)

	if len(args) < 2:
		print "ERROR: you must specify a .pickle file as second argument"
		sys.exit(1)
	
	with nijmetrics.Tee(tee):
		if args[0] == 'images':
			app = nijmetrics.QMapToImage(args[1])
		elif args[0] == 'csv':
			app = nijmetrics.QMapToCSV(args[1])
		elif args[0] == 'coherency':
			app = nijmetrics.Coherency(args[1])
		else:
			print "ERROR: %s is not a valid action. Try --help" % args[0]
			sys.exit(1)
			
		sys.exit(0 if app.run() else 1)
	